#include "neuroninterz.h"

NeuronInterz::NeuronInterz()
{

// 	maxSynapses			= 150;

	isInhibitory			= false;

	// input weight range, 5 = -5 <-> +5	-> 10 in total because 0 will be excluded
// 	dendridicBranches		= 10;

	// processing
// 	firingThreshold			= 30.0f;
	potential			= 0.0f;
	potentialDecay			= 0.95f;

	// output
	output				= 0;
	waitoutput			= 0;

	// plasticity up & down
	isPlastic			= false;
// 	plasticityStrengthen		= 1.0f+(1.0f/100.0f);
// 	plasticityWeaken		= 1.0f-(1.0f/1000.0f);

	// SPECIALO optional reference that makes this a MOTOR neuron, but it depends on not being defined
	isMotor				= false;
	motorFunc			= 0;


        settings = Settings::Instance();	
	pill_maxtrip = settings->getCVarPtr("pill_maxtrip");
}

void NeuronInterz::process(float* tripping)
{
//	if (tripping != 0) {
//		printf("tripping value is %f\n",tripping);
//		}
	// potential decay
	potential = potential * potentialDecay;

	// make every connection do it's influence on the neuron's total potential
	unsigned int ssize = Synapses.size();
	for ( unsigned int i=0; i < ssize; i++ )
	{
		// lower synaptic weights
		if (isPlastic)
			Synapses[i].weight = Synapses[i].weight * plasticityWeaken;

		if ( *Synapses[i].ref != 0 )
			potential += (Synapses[i].weight * Synapses[i].dendriteBranch * *Synapses[i].ref);
	}

	float alien_ai_fleet_seed;
	if ( isInhibitory )
	{
		if (tripping[1]) { // inhibitory agonism
			alien_ai_fleet_seed = tripping[1];
			potential = potential - (tripping[1] / (1000.0f - ((tripping[4] / *pill_maxtrip)*1000.0f)));
			tripping[1] = alien_ai_fleet_seed;                // whats happening to value of tripping that i need to do this (flamoot)
		}
		
		if (tripping[0]) { // inhibitory antagonism
			alien_ai_fleet_seed = tripping[0];
			potential = potential + (tripping[0] / (1000.0f - ((tripping[4] / *pill_maxtrip)*1000.0f)));
			tripping[0] = alien_ai_fleet_seed;
		}


//		printf("tripping %f %f\n",tripping,potential);
//		cout << "I" << potential << "\n";

		// do we spike/fire
		if ( potential <= -1.0f * firingThreshold )
		{
			// reset neural potential
			potential = 0.0f;
	
			// fire the neuron
			waitoutput = -1;

			// PLASTICITY: if neuron & synapse fire together, the synapse strenghtens
			if (isPlastic)
			{
				for ( unsigned int i=0; i < ssize; i++ )
				{
					// if synapse fired, strenghten the weight
					if ( (*Synapses[i].ref < 0.0f && Synapses[i].weight > 0.0f) || (*Synapses[i].ref > 0.0f && Synapses[i].weight < 0.0f) )
					{
// 						cerr << endl << "Inhibitory firing" << endl << "synref: " << *Synapses[i].ref << endl << "pre weight:  " << Synapses[i].weight << endl;
						Synapses[i].weight = Synapses[i].weight * plasticityStrengthen;
// 						cerr << "post weight: " << Synapses[i].weight << endl;
					}

					// if weight > max back to max
 					if	( Synapses[i].weight > 5.0f ) Synapses[i].weight = 5.0f;
 					else if ( Synapses[i].weight <-5.0f ) Synapses[i].weight =-5.0f;
				}
			}
		}
		// don't fire the neuron
		else
		{
			waitoutput = 0;
			// reset potential if < 0
			if ( potential > 0.0f ) potential = 0.0f;
		}
	}
	else
	{
		if (tripping[3]) { // excitatory agonism
			alien_ai_fleet_seed = tripping[3];
			potential = potential + (tripping[3] / (1000.0f - ((tripping[4] / *pill_maxtrip)*1000.0f)));
			tripping[3] = alien_ai_fleet_seed;
		}
		
		if (tripping[2]) { // excitatory antagonism
			alien_ai_fleet_seed = tripping[2];
			potential = potential - (tripping[2] / (1000.0f - ((tripping[4] / *pill_maxtrip)*1000.0f)));
			tripping[2] = alien_ai_fleet_seed;
		}		


//		printf("tripping %f %f\n",tripping,potential);
//		cout << "E" << potential << "\n";

		// do we spike/fire
		if ( potential >= firingThreshold )
		{
			// reset neural potential
			potential = 0.0f;
	
			// fire the neuron
			waitoutput = 1;

			// PLASTICITY: if neuron & synapse fire together, the synapse strenghtens
			if (isPlastic)
			{
				for ( unsigned int i=0; i < ssize; i++ )
				{
					// if synapse fired, strenghten the weight
					if ( (*Synapses[i].ref > 0.0f && Synapses[i].weight > 0.0f) || (*Synapses[i].ref < 0.0f && Synapses[i].weight < 0.0f) )
					{
// 						cerr << endl << "Excititory firing" << endl << "synref: " << *Synapses[i].ref << endl << "pre weight:  " << Synapses[i].weight << endl;
						Synapses[i].weight = Synapses[i].weight * plasticityStrengthen;
// 						cerr << "post weight: " << Synapses[i].weight << endl;
					}

					// if weight > max back to max
 					if	( Synapses[i].weight > 5.0f ) Synapses[i].weight = 5.0f;
 					else if ( Synapses[i].weight <-5.0f ) Synapses[i].weight =-5.0f;
				}
			}
		}
		// don't fire the neuron
		else
		{
			waitoutput = 0;
			// reset potential if < 0
			if ( potential < 0.0f ) potential = 0.0f;
		}
		//if (tripping) { tripping -= 0.00001f; }
	}
//	return tripping; // unchanged anyway and is a vector now so i dont know how anymore
}

void NeuronInterz::connec( float *output, unsigned int dendriteBranch, float synapticWeight )
//void NeuronInterz::connec( int *output, unsigned int dendriteBranch, float synapticWeight )
{
		Synapse s;
		s.ref = output;
		s.dendriteBranch = dendriteBranch;
		s.weight = synapticWeight;

		Synapses.push_back( s );

//		cerr << "I: " << Synapses.size()-1 << "=" << *Synapses[Synapses.size()-1].ref << "(" << Synapses[Synapses.size()-1].ref << ")";
//		cerr << *Synapses[0].ref << endl;
}

NeuronInterz::~NeuronInterz()
{
//	for ( unsigned int i=0; i < inputs.size(); i++ ) delete inputs[i];
}


